/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *
 *  utility.h - Definition file for utility functions
 *
 ******************************************************************************/

#ifndef UTILITY_H_
#define UTILITY_H_

/*****************************************************************************/

// change to reflect firmware revision
#define FW_VERSION 1.0
// count to toggle status LED and force update
#define LED_TIMEOUT 10000

//
#define	DEFAULT_IFS				0
#define	DEFAULT_SENSE_RESISTOR	0.25
#define	DEFAULT_VREF_VAL		300

//GPIO Port Definitions
#define nSLEEP      BIT2    // P2.2
#define AIN1        BIT3    // P2.3
#define AIN2        BIT4    // P2.4
#define BIN2        BIT1    // P4.1
#define BIN1        BIT2    // P4.2
#define VREF        BIT6    // P6.6
#define nFault      BIT4    // P2.2

// GPIO Port 6 Definitions
#define STATUS		BIT5	// P6.5

// Custom Types
typedef enum {false, true} boolean;
typedef enum {low, high} gpio;

/*****************************************************************************/

// Declare Global Variables

//***********************************************************************************
// Global Variables for GUI Composer                                                         *
//***********************************************************************************
extern unsigned int  G_PWM_AIN1;
extern unsigned int  G_PWM_AIN2;
extern unsigned int  G_PWM_BIN1;
extern unsigned int  G_PWM_BIN2;

extern unsigned int  G_Normal_A;
extern unsigned int  G_Normal_B;
extern unsigned int  G_Parallel;
extern unsigned int  G_Stepper;

// GPIO
extern gpio G_WAKE;				// Wake device		0 - Sleep		1 - Wake
extern gpio G_DIRECTION;		// Direction		0 - Forward		1 - Reverse
extern gpio G_FAULT;			// Fault Status		0 - Fault		1 - OK
extern gpio G_RUN;

extern unsigned int G_Mode;

// PWM Generation

extern unsigned long G_VREF_VAL;	// scale for current
extern unsigned int G_PPS_Value;

extern float  G_VREF;           // VREF
extern float  G_IFS;			// Full scale current
extern float  G_IRSENSE;		// sense value resistor
extern float  G_FIRMWARE_VERSION; // keep track of firmware

// use 0/1 and shift by this value when
// necessary
unsigned int  LED_TOGGLE_CNT;	// count to determine if LED should be toggled

/*****************************************************************************/
// Function Declarations

// Main Functions
void Initialize();
void UpdateSystem();

// GPIO/GUI Functions
void SetDefaultGUIVals();
void SetDefaultGPIOVals();


#endif /* UTILITY_H_ */
